@@ -23,9 +23,11 @@
 
 #include <algorithm>
 #include "KeyManager.hpp"
-#include "Interface.hpp"
+#include "Interpreter.hpp"
+#include "Common.hpp"
 #include "Context.hpp"
 #include <qapplication.h>
+#include <QtCore>
 #include <iostream>
 
 #define TAB_WIDTH 8
@@ -45,12 +47,13 @@
 // Maximum allowed number of columns in the text window
 #define MAXCOLS 256
 
+#define LINELEN 65536
 
 // ----------------------------------------------------------------------------
 // KeyManager
 // ----------------------------------------------------------------------------
 
-KeyManager::KeyManager() : Interface() {
+KeyManager::KeyManager()  {
   cutbuf = "";
   linelen = 1000;
   ntotal = 0;
@@ -65,9 +68,21 @@ KeyManager::KeyManager() : Interface() {
   enteredLinesEmpty = true;
   ReplacePrompt("");
   loopactive = 0;
-  m_loop = new QEventLoop;
-  lineData = new char[4096];
+  lineData = new char[LINELEN];
   ResetLineBuffer();
+  context = NULL;
+  QSettings settings("FreeMat", "FreeMat");
+  QStringList historyList = settings.value("interpreter/history").toStringList();
+  for (int i=0;i<historyList.size();i++) 
+    history.push_back(historyList[i].toStdString());
+}
+
+Context* KeyManager::GetCompletionContext() {
+  return context;
+}
+
+void KeyManager::SetCompletionContext(Context* ctxt) {
+  context = ctxt;
 }
 
 int KeyManager::getTerminalWidth() {
@@ -75,6 +90,7 @@ int KeyManager::getTerminalWidth() {
 }
 
 void KeyManager::SetTermWidth(int w) {
+  emit UpdateTermWidth(w);
   ncolumn = w;
   Redisplay();
 }
@@ -118,7 +134,7 @@ int KeyManager::DisplayedCharWidth(char c, int aterm_curpos) {
 
 // Return the number of terminal characters needed to display a
 // given substring.
-int KeyManager::DisplayedStringWidth(std::string s, int nc, int aterm_curpos) {
+int KeyManager::DisplayedStringWidth(string s, int nc, int aterm_curpos) {
   int slen=0;   /* The displayed number of characters */
   int i;
   /*
@@ -134,7 +150,7 @@ int KeyManager::DisplayedStringWidth(std::string s, int nc, int aterm_curpos) {
   return slen;
 }
 
-void KeyManager::InsertString(int pos, std::string s) {
+void KeyManager::InsertString(int pos, string s) {
   int len = s.size();
   for (int i=4096-len;i>pos;i--)
     lineData[i] = lineData[i-len];
@@ -163,40 +179,40 @@ int KeyManager::BuffCurposToTermCurpos(int n) {
 }
 
 void KeyManager::Redisplay() {
-   /*
-    * Keep a record of the current cursor position.
-    */
-   int sbuff_curpos = buff_curpos;
-   /*
-    * Move the cursor to the start of the terminal line, and clear from there
-    * to the end of the display.
-    */
-   SetTermCurpos(0);
-   emit ClearEOD();
-   /*
-    * Nothing is displayed yet.
-    */
-   term_len = 0;
-   /*
-    * Display the current prompt.
-    */
-   DisplayPrompt();
-   /*
-    * Render the part of the line that the user has typed in so far.
-    */
-   OutputString(lineData,'\0');
-   /*
-    * Restore the cursor position.
-    */
-   PlaceCursor(sbuff_curpos);
+  /*
+   * Keep a record of the current cursor position.
+   */
+  int sbuff_curpos = buff_curpos;
+  /*
+   * Move the cursor to the start of the terminal line, and clear from there
+   * to the end of the display.
+   */
+  SetTermCurpos(0);
+  emit ClearEOD();
+  /*
+   * Nothing is displayed yet.
+   */
+  term_len = 0;
+  /*
+   * Display the current prompt.
+   */
+  DisplayPrompt();
+  /*
+   * Render the part of the line that the user has typed in so far.
+   */
+  OutputString(lineData,'\0');
+  /*
+   * Restore the cursor position.
+   */
+  PlaceCursor(sbuff_curpos);
 }
 
 void KeyManager::setTerminalWidth(int w) {
-  std::cout << "Set terminal width " << w << "\n";
+  //  cout << "Set terminal width " << w << "\n";
   ncolumn = w; 
 }
 
-void KeyManager::ReplacePrompt(std::string aprompt) {
+void KeyManager::ReplacePrompt(string aprompt) {
   prompt = aprompt;
   prompt_len = DisplayedStringWidth(aprompt,-1,0);
 }
@@ -331,13 +347,13 @@ void KeyManager::AddCharToLine(char c) {
      * Redraw the line from the cursor position to the end of the line,
      * and move the cursor to just after the added character.
      */
-    OutputString(std::string(lineData+sbuff_curpos), '\0');
+    OutputString(string(lineData+sbuff_curpos), '\0');
     SetTermCurpos(sterm_curpos + width);
     /*
      * Are we overwriting an existing character?
      */
-   } else {
-     /*
+  } else {
+    /*
      * Get the widths of the character to be overwritten and the character
      * that is going to replace it.
      */
@@ -354,7 +370,7 @@ void KeyManager::AddCharToLine(char c) {
      * with spaces.
      */
     if(old_width > width) {
-      OutputString(std::string(lineData+sbuff_curpos), '\0');
+      OutputString(string(lineData+sbuff_curpos), '\0');
       /*
        * Clear to the end of the terminal.
        */
@@ -373,7 +389,7 @@ void KeyManager::AddCharToLine(char c) {
        * Redraw the line from the cursor position to the end of the line,
        * and move the cursor to just after the added character.
        */
-      OutputString(std::string(lineData+sbuff_curpos), '\0');
+      OutputString(string(lineData+sbuff_curpos), '\0');
       SetTermCurpos(sterm_curpos + width);
       buff_curpos++;
       /*
@@ -391,7 +407,7 @@ void KeyManager::AddCharToLine(char c) {
        */
       OutputChar(c, lineData[buff_curpos]);
     };
-   };
+  };
 }
 
 int KeyManager::DisplayPrompt() {
@@ -485,7 +501,7 @@ void KeyManager::OutputChar(char c, char pad) {
   };
 }
 
-void KeyManager::OutputString(std::string st, char pad) {
+void KeyManager::OutputString(string st, char pad) {
   if (st.size() == 0) return;
   for(unsigned int i=0;i<st.size()-1;i++)
     OutputChar(st[i],st[i+1]);
@@ -510,16 +526,19 @@ void KeyManager::NewLine() {
   AddHistory(lineData);
   PlaceCursor(ntotal);
   emit OutputRawString("\r\n");
-  ExecuteLine(std::string(lineData) + "\n");
+  emit ExecuteLine(string(lineData) + "\n");
+  ReplacePrompt("");
   ResetLineBuffer();
   DisplayPrompt();
 }
 
-extern bool InterruptPending;
+void KeyManager::Ready() {
+  ReplacePrompt("--> ");
+  Redisplay();
+}
 
 void KeyManager::RegisterInterrupt() {
-  InterruptPending  = true;
-  emit Interrupt();
+  sigInterrupt(0);
 }
 
 void KeyManager::CursorLeft() {
@@ -544,8 +563,8 @@ void KeyManager::ForwardDeleteChar() {
   DeleteChars(1,0);
 }
 
-  // Delete nc characters starting from the one under the cursor.
-  // Optionally copy the deleted characters to the cut buffer.
+// Delete nc characters starting from the one under the cursor.
+// Optionally copy the deleted characters to the cut buffer.
 void KeyManager::DeleteChars(int nc, int cut) {
   /*
    * If there are fewer than nc characters following the cursor, limit
@@ -558,7 +577,7 @@ void KeyManager::DeleteChars(int nc, int cut) {
    */
   if(cut) {
     // Fixme
-    //    cutbuf = std::string(line,buff_curpos,nc);
+    //    cutbuf = string(line,buff_curpos,nc);
   }
   /*
    * Nothing to delete?
@@ -575,7 +594,7 @@ void KeyManager::DeleteChars(int nc, int cut) {
   /*
    * Redraw the remaining characters following the cursor.
    */
-  OutputString(std::string(lineData+buff_curpos), '\0');
+  OutputString(string(lineData+buff_curpos), '\0');
   /*
    * Clear to the end of the terminal.
    */
@@ -591,7 +610,7 @@ void KeyManager::EndOfLine() {
 }
 
 void KeyManager::KillLine() {
-  cutbuf = std::string(lineData+buff_curpos);
+  cutbuf = string(lineData+buff_curpos);
   ntotal = buff_curpos;
   memset(lineData+ntotal,0,4096-ntotal);
   TruncateDisplay();
@@ -600,7 +619,7 @@ void KeyManager::KillLine() {
 
 void KeyManager::HistorySearchBackward() {
   if (last_search != keyseq_count-1)
-    SearchPrefix(std::string(lineData),buff_curpos);
+    SearchPrefix(string(lineData),buff_curpos);
   last_search = keyseq_count;
   HistoryFindBackwards();
   ntotal = strlen(lineData);
@@ -610,7 +629,7 @@ void KeyManager::HistorySearchBackward() {
 
 void KeyManager::HistorySearchForward() {
   if (last_search != keyseq_count-1)
-    SearchPrefix(std::string(lineData),buff_curpos);
+    SearchPrefix(string(lineData),buff_curpos);
   last_search = keyseq_count;
   HistoryFindForwards();
   ntotal = strlen(lineData);
@@ -618,13 +637,13 @@ void KeyManager::HistorySearchForward() {
   Redisplay();
 }
 
-void KeyManager::SearchPrefix(std::string aline, int aprefix_len) {
-  prefix = std::string(aline,0,aprefix_len);
+void KeyManager::SearchPrefix(string aline, int aprefix_len) {
+  prefix = string(aline,0,aprefix_len);
   prefix_len = aprefix_len;
   startsearch = history.size();
 }
 
-void KeyManager::AddHistory(std::string mline) {
+void KeyManager::AddHistory(string mline) {
   prefix = "";
   prefix_len = 0;
   history.push_back(mline);
@@ -647,6 +666,7 @@ void KeyManager::HistoryFindForwards() {
     ResetLineBuffer();
     return;
   }
+  memset(lineData,0,LINELEN);
   strcpy(lineData,history[i].c_str());
   startsearch = i;
 }
@@ -662,6 +682,7 @@ void KeyManager::HistoryFindBackwards() {
     if (!found) i--;
   }
   if (!found) return;
+  memset(lineData,0,LINELEN);
   strcpy(lineData,history[i].c_str());
   startsearch = i;
 }
@@ -679,14 +700,14 @@ void KeyManager::HistoryFindBackwards() {
  *  return   int     0 - OK.
  *                   1 - Insufficient room.
  */
-void KeyManager::AddStringToLine(std::string s) {
+void KeyManager::AddStringToLine(string s) {
   int buff_slen;   /* The length of the string being added to line[] */
   int term_slen;   /* The length of the string being written to the terminal */
   int sbuff_curpos; /* The original value of gl->buff_curpos */
   int sterm_curpos; /* The original value of gl->term_curpos */
   /*
- * Keep a record of the current cursor position.
- */
+   * Keep a record of the current cursor position.
+   */
   sbuff_curpos = buff_curpos;
   sterm_curpos = term_curpos;
   /*
@@ -718,7 +739,7 @@ void KeyManager::AddStringToLine(std::string s) {
    * Write the modified part of the line to the terminal, then move
    * the terminal cursor to the end of the displayed input string.
    */
-  OutputString(std::string(lineData+sbuff_curpos), '\0');
+  OutputString(string(lineData+sbuff_curpos), '\0');
   SetTermCurpos(sterm_curpos + term_slen);
 }
 
@@ -729,7 +750,7 @@ void KeyManager::Yank() {
   AddStringToLine(cutbuf);
 }
 
-void KeyManager::ListCompletions(std::vector<std::string> completions) {
+void KeyManager::ListCompletions(vector<string> completions) {
   int maxlen;    /* The length of the longest matching string */
   int width;     /* The width of a column */
   int ncol;      /* The number of columns to list */
@@ -791,12 +812,12 @@ void KeyManager::ListCompletions(std::vector<std::string> completions) {
   };
 }
 
-std::string GetCommonPrefix(std::vector<std::string> matches,
-			    std::string tempstring) {
+string GetCommonPrefix(vector<string> matches,
+			    string tempstring) {
   unsigned int minlength;
   unsigned int prefixlength;
   bool allmatch;
-  std::string templ;
+  string templ;
   unsigned int i, j;
 
   minlength = matches[0].size();
@@ -809,18 +830,19 @@ std::string GetCommonPrefix(std::vector<std::string> matches,
     j = 0;
     allmatch = true;
     while (allmatch && (j<prefixlength)) {
-      std::string mtch(matches[i]);
+      string mtch(matches[i]);
       allmatch = (mtch[j] == templ[j]);
       if (allmatch) j++;
     }
     prefixlength = (j < prefixlength) ? j : prefixlength;
   }
   if (prefixlength <= tempstring.length())
-    return (std::string(""));
+    return (string(""));
   else
     return(templ.substr(tempstring.length(),prefixlength-tempstring.length()));
 }
 
+
 void KeyManager::CompleteWord() {
   int redisplay=0;        /* True if the whole line needs to be redrawn */
   int suffix_len;         /* The length of the completion extension */
@@ -829,7 +851,7 @@ void KeyManager::CompleteWord() {
                           /*  total length of the line. */
   int buff_pos;           /* The buffer index at which the completion is */
                           /*  to be inserted. */
-  std::vector<std::string> matches;
+  vector<string> matches;
   redisplay = 1;
   /*
    * Get the cursor position at which the completion is to be inserted.
@@ -838,7 +860,7 @@ void KeyManager::CompleteWord() {
   /*
    * Perform the completion.
    */
-  std::string tempstring;
+  string tempstring;
   matches = GetCompletions(lineData, buff_curpos, tempstring);
   if(matches.size() == 0) {
     emit OutputRawString("\r\n");
@@ -859,7 +881,7 @@ void KeyManager::CompleteWord() {
     /*
      * Find the common prefix
      */
-    std::string prefix;
+    string prefix;
     prefix = GetCommonPrefix(matches, tempstring);
     /*
      * Get the length of the suffix and any continuation suffix to add to it.
@@ -881,7 +903,7 @@ void KeyManager::CompleteWord() {
 	/*
 	 * Make room to insert the filename extension.
 	 */
-	InsertString(buff_curpos,std::string(prefix,0,nextra));
+	InsertString(buff_curpos,string(prefix,0,nextra));
 	/*
 	 * Record the increased length of the line.
 	 */
@@ -897,7 +919,7 @@ void KeyManager::CompleteWord() {
 	 */
 	if(!redisplay) {
 	  TruncateDisplay();
-	  OutputString(std::string(lineData+buff_pos), '\0');
+	  OutputString(string(lineData+buff_pos), '\0');
 	  PlaceCursor(buff_curpos);
 	  return;
 	};
@@ -951,7 +973,7 @@ void KeyManager::OnChar( int c ) {
   case KM_CTRLA:
     BeginningOfLine();
     break;
-  case KM_CTRLC:
+  case KM_CTRLZ:
     RegisterInterrupt();
     break;
   case KM_CTRLE:
@@ -982,49 +1004,8 @@ void KeyManager::OnChar( int c ) {
   }
 }
 
-std::string TranslateString(std::string x) {
-  std::string y(x);
-  unsigned int n;
-  n = 0;
-  while (n<y.size()) {
-    if (y[n] == '\n') 
-      y.insert(n++,"\r");
-    n++;
-  }
-  return y;
-}
-
-void KeyManager::CWDChanged() {
-  emit UpdateCWD();
-}
-
-void KeyManager::outputMessage(std::string msg) {
-  std::string msg2(TranslateString(msg));
-  emit OutputRawString(msg2);
-}
-
-void KeyManager::errorMessage(std::string msg) {
-  std::string msg2(TranslateString(msg));
-  emit OutputRawString("Error: " + msg2 + "\r\n");
-}
-
-void KeyManager::warningMessage(std::string msg) {
-  std::string msg2(TranslateString(msg));
-  emit OutputRawString("Warning: " + msg2 + "\r\n");
-}
-
-void KeyManager::ExecuteLine(std::string mline) {
-  enteredLines.push_back(mline);
-  ReplacePrompt("");
-  enteredLinesEmpty = false;
-  if (loopactive) {
-    loopactive--;
-    m_loop->exit();
-  }
-}
-
 void KeyManager::QueueString(QString t) {
-  std::string g(t.toStdString());
+  string g(t.toStdString());
   AddStringToLine(g);
 }
 
@@ -1034,51 +1015,51 @@ void KeyManager::QueueMultiString(QString t) {
     return;
   }
   QStringList tlist(t.split("\n"));
-  if (tlist.size() > 0)
-    AddStringToLine(tlist[0].toStdString()); 
-  emit OutputRawString("\r\n");
-  ResetLineBuffer();
-  ReplacePrompt("");
-  for (int i=1;i<tlist.size();i++)
-    emit OutputRawString(tlist[i].toStdString() + "\r\n");
-  for (int i=0;i<tlist.size();i++) {
-    enteredLines.push_back(tlist[i].toStdString());
-    AddHistory(tlist[i].toStdString());
+  for (int i=0;i<tlist.size()-1;i++) {
+    string t(tlist[i].toStdString());
+    emit OutputRawString(t+"\r\n");
+    emit ExecuteLine(t+"\n");
+    AddHistory(t);
   }
-  enteredLinesEmpty = false;
-  if (loopactive) {
-    loopactive--;
-    m_loop->exit();
+  if (t.endsWith('\n')) {
+    string t(tlist.back().toStdString());
+    emit OutputRawString(t+"\r\n");
+    emit ExecuteLine(t+"\n");
+    AddHistory(t);
+  }  else {
+    QueueString(tlist.back());
   }
+  return;
 }
 
 void KeyManager::QueueCommand(QString t) {
   QueueString(t);
+  AddHistory(t.toStdString());
   emit OutputRawString("\r\n");
-  ExecuteLine(std::string(lineData)+"\n");
+  emit ExecuteLine(string(lineData)+"\n");
   ResetLineBuffer();
   DisplayPrompt();
 }
 
 void KeyManager::QueueSilent(QString t) {
-  ExecuteLine(t.toStdString()+"\n");
+  emit ExecuteLine(t.toStdString()+"\n");
 }
 
-char* KeyManager::getLine(std::string aprompt) {
-  emit UpdateVariables();
-  ReplacePrompt(aprompt);
-  DisplayPrompt();
-  while (enteredLinesEmpty) {
-    loopactive++;
-    m_loop->exec();
-  }
-  std::string theline(enteredLines.front());
-  enteredLines.pop_front();
-  enteredLinesEmpty = (enteredLines.empty());
-  char *cp;
-  cp = strdup(theline.c_str());
-  return cp;
-}
+//char* KeyManager::getLine(string aprompt) {
+//  emit UpdateVariables();
+//  ReplacePrompt(aprompt);
+//  DisplayPrompt();
+//  while (enteredLinesEmpty) {
+//    loopactive++;
+//    m_loop->exec();
+//  }
+//  string theline(enteredLines.front());
+//  enteredLines.pop_front();
+//  enteredLinesEmpty = (enteredLines.empty());
+//  char *cp;
+//  cp = strdup(theline.c_str());
+//  return cp;
+//}
 
 void KeyManager::RegisterTerm(QObject* term) {
   connect(this,SIGNAL(MoveDown()),term,SLOT(MoveDown()));
@@ -1088,15 +1069,120 @@ void KeyManager::RegisterTerm(QObject* term) {
   connect(this,SIGNAL(ClearEOL()),term,SLOT(ClearEOL()));
   connect(this,SIGNAL(ClearEOD()),term,SLOT(ClearEOD()));
   connect(this,SIGNAL(MoveBOL()),term,SLOT(MoveBOL()));
-  connect(this,SIGNAL(OutputRawString(std::string)),term,SLOT(OutputRawString(std::string)));
+  connect(this,SIGNAL(OutputRawString(string)),term,SLOT(OutputRawString(string)));
   connect(term,SIGNAL(OnChar(int)),this,SLOT(OnChar(int)));
   connect(term,SIGNAL(SetTextWidth(int)),this,SLOT(SetTermWidth(int)));  
 }
 
 void KeyManager::ContinueAction() {
-  ExecuteLine("return\n");
+  emit ExecuteLine("return\n");
 }
 
 void KeyManager::StopAction() {
-  ExecuteLine("retall\n");  
+  emit ExecuteLine("retall\n");  
+}
+
+void KeyManager::DbStepAction() {
+  emit ExecuteLine("dbstep\n");
+}
+
+void KeyManager::DbTraceAction() {
+  emit ExecuteLine("dbtrace\n");
+}
+
+void KeyManager::SetPrompt(string txt) {
+  ReplacePrompt(txt);
+  Redisplay();
+  emit UpdateVariables();
+}
+
+/*.......................................................................
+ * Search backwards for the potential start of a filename. This
+ * looks backwards from the specified index in a given string,
+ * stopping at the first unescaped space or the start of the line.
+ *
+ * Input:
+ *  string  const char *  The string to search backwards in.
+ *  back_from      int    The index of the first character in string[]
+ *                        that follows the pathname.
+ * Output:
+ *  return        char *  The pointer to the first character of
+ *                        the potential pathname, or NULL on error.
+ */
+static char *start_of_path(const char *string, int back_from)
+{
+  int i, j;
+  /*
+   * Search backwards from the specified index.
+   */
+  for(i=back_from-1; i>=0; i--) {
+    int c = string[i];
+    /*
+     * Stop on unescaped spaces.
+     */
+    if(isspace((int)(unsigned char)c)) {
+      /*
+       * The space can't be escaped if we are at the start of the line.
+       */
+      if(i==0)
+	break;
+      /*
+       * Find the extent of the escape characters which precedes the space.
+       */
+      for(j=i-1; j>=0 && string[j]=='\\'; j--)
+	;
+      /*
+       * If there isn't an odd number of escape characters before the space,
+       * then the space isn't escaped.
+       */
+      if((i - 1 - j) % 2 == 0)
+	break;
+    } 
+    else if (!isalpha(c) && !isdigit(c) && (c != '_') && (c != '.') && (c != '\\') && (c != '/'))
+      break;
+  };
+  return (char *)string + i + 1;
+}
+
+vector<string> KeyManager::GetCompletions(string line, 
+					  int word_end, 
+					  string &matchString) {
+  vector<string> completions;
+  if (!context->getMutex()->tryLock()) return completions;
+  QMutexLocker lock(context->getMutex());
+  context->getMutex()->unlock();
+  /*
+   * Find the start of the filename prefix to be completed, searching
+   * backwards for the first unescaped space, or the start of the line.
+   */
+  char *start = start_of_path(line.c_str(), word_end);
+  char *tmp;
+  int mtchlen;
+  mtchlen = word_end - (start-line.c_str());
+  tmp = (char*) malloc(mtchlen+1);
+  memcpy(tmp,start,mtchlen);
+  tmp[mtchlen] = 0;
+  matchString = string(tmp);
+  
+  /*
+   *  the preceeding character was not a ' (quote), then
+   * do a command expansion, otherwise, do a filename expansion.
+   */
+  if (!context) return completions;
+  if (start[-1] != '\'') {
+    vector<string> local_completions;
+    vector<string> global_completions;
+    int i;
+    local_completions = context->getCurrentScope()->getCompletions(string(start));
+    global_completions = context->getGlobalScope()->getCompletions(string(start));
+    for (i=0;i<local_completions.size();i++)
+      completions.push_back(local_completions[i]);
+    for (i=0;i<global_completions.size();i++)
+      completions.push_back(global_completions[i]);
+  }
+  stringVector comp(GetCompletionList(tmp));
+  for (int i=0;i<comp.size();i++)
+    completions.push_back(comp[i]);
+  sort(completions.begin(),completions.end());
+  return completions;
 }
