@@ -23,19 +23,17 @@
 #include <string>
 #include <vector>
 
-namespace FreeMat {
-  class PathSearcher{
-    std::string path;
-    std::vector<std::string> pathList;
-  public:
-    PathSearcher(std::string mpath);
-    std::string ResolvePath(std::string fname);
-  };
-  bool FileExists(std::string filename);
-  std::string GetPathOnly(std::string a);
-  std::string GetFilenameOnly(std::string a);
-  std::string CheckEndSlash(std::string a);
-}
+class PathSearcher{
+  std::string path;
+  std::vector<std::string> pathList;
+public:
+  PathSearcher(std::string mpath);
+  std::string ResolvePath(std::string fname);
+};
+bool FileExists(std::string filename);
+std::string GetPathOnly(std::string a);
+std::string GetFilenameOnly(std::string a);
+std::string CheckEndSlash(std::string a);
 
 #endif  
 
