@@ -41,10 +41,10 @@ tagChar* tagArray::buffer() {
 }
 
 tagChar& tagArray::operator[](int pos) {
-  if (pos < 0)
-    qDebug("Negative array access\n");
-  if (pos >= m_length)
-    qDebug("Exceed array index\n");
+  //   if (pos < 0)
+  //     qDebug("Negative array access\n");
+  //   if (pos >= m_length)
+  //     qDebug("Exceed array index\n");
   return m_buffer[pos];
 }
 
@@ -171,6 +171,7 @@ void QTTerm::ScrollLineUp() {
 
 void QTTerm::ScrollLineDown() {
   m_scrollbar->setValue(m_scrollbar->value()+1);
+
 }
 
 int QTTerm::GetWidth() {
@@ -307,12 +308,12 @@ void QTTerm::keyPressEvent(QKeyEvent *e) {
   if (keycode == Qt::Key_Left)
     emit OnChar(KM_LEFT);
   CTRLKEY('Z')
-    emit OnChar(KM_CTRLC);
+    emit OnChar(KM_CTRLK);
   CTRLKEY('A')
     emit OnChar(KM_CTRLA);
   CTRLKEY('D')
-    emit OnChar(KM_CTRLD);
-  CTRLKEY('E')
+    emit OnChar(KM_CTRLD); 
+ CTRLKEY('E')
     emit OnChar(KM_CTRLE);
   CTRLKEY('K')
     emit OnChar(KM_CTRLK);
@@ -456,6 +457,7 @@ void QTTerm::MoveBOL() {
   SetCursor(0,m_cursor_y);
 }
 
+
 void QTTerm::ClearEOL() {
   for (int i=m_cursor_x;i<m_width;i++)
     m_surface[i + m_cursor_y*m_width] = tagChar(' ');
@@ -466,7 +468,7 @@ void QTTerm::ClearEOD() {
     m_surface[i] = tagChar(' ');
 }
 
-void QTTerm::OutputRawString(std::string txt) {
+void QTTerm::OutputRawString(string txt) {
   PutString(txt);
 }
 
@@ -583,7 +585,7 @@ void QTTerm::setScrollbar(int val) {
     SetScrollBarValue(val);
 }
 
-void QTTerm::PutString(std::string txt) {
+void QTTerm::PutString(string txt) {
   m_firsttime = false;
   if (m_scrolling) {
     setScrollbar(m_history_lines);
