@@ -34,10 +34,10 @@
 #include "Context.hpp"
 #include "helpgen.hpp"
 #include <QtGui>
+#include "Common.hpp"
 
 QTextEdit *m_text;
 
-QString output;
 QStringList input;
 QStringList generatedFileList;
 QMap<QString, QString> section_descriptors;
@@ -46,6 +46,33 @@ int moduledepth = 0;
 QString sourcepath;
 
 
+HelpHandler::HelpHandler(Interpreter *interp, 
+			 QStringList cmds) {
+  m_interp = interp;
+  m_cmds = cmds;
+}
+
+void HelpHandler::AddOutputString(string txt) {
+  m_text += txt;
+}
+
+void HelpHandler::SetPrompt(string prompt) {
+  m_text += prompt;
+  m_interp->ExecuteLine(m_cmds.front().toStdString());
+  m_text += m_cmds.front().toStdString();
+  m_cmds.erase(m_cmds.begin());
+}
+
+void HelpHandler::DoGraphicsCall(FuncPtr f, ArrayVector m, int narg) { 
+  try {
+    ArrayVector n(f->evaluateFunction(m_interp,m,narg));
+    m_interp->RegisterGfxResults(n);
+  } catch (Exception& e) {
+    m_interp->RegisterGfxError(e.getMessageCopy());
+  }
+}
+
+
 void TermOutputText(QString str) {
   //  m_text->textCursor().movePosition(QTextCursor::End);
   m_text->insertPlainText(str);
@@ -55,52 +82,30 @@ void TermOutputText(QString str) {
 }
 
 
-void OutputHelper::AddOutputString(std::string txt) {
-  output += QString::fromStdString(txt);
-}
-
-void OutputHelper::MoveDown() {
-  output += "\n";
-}
-
-
-class HelpTerminal : public KeyManager {
-public:
-  HelpTerminal() {}
-  virtual ~HelpTerminal() {}
-  virtual char* getLine(std::string aprompt) {
-    if (input.empty()) return 0;
-    QString txt(input[0]);
-    input.removeFirst();
-    char *rettxt = strdup(qPrintable(txt));
-    if (!input.empty())
-      output += aprompt.c_str() + txt;
-    return (strdup(qPrintable(txt)));      
-  }
-};
-
-HelpTerminal *m_term;
-using namespace FreeMat;
 Context *context;
-OutputHelper *m_hlp;
+Interpreter *m_interp;
+HelpHandler *m_hlp;
 
-WalkTree* GetInterpreter() {
-  m_term = new HelpTerminal;
-  m_hlp = new OutputHelper;
-  QObject::connect(m_term,SIGNAL(MoveDown()),
-		   m_hlp,SLOT(MoveDown()));
-  QObject::connect(m_term,SIGNAL(OutputRawString(std::string)),
-		   m_hlp,SLOT(AddOutputString(std::string)));
-  LoadModuleFunctions(context);
-  LoadClassFunction(context);
-  LoadCoreFunctions(context);
-  LoadFNFunctions(context);
-  LoadHandleGraphicsFunctions(context);  
-  m_term->setContext(context);
-  QString pth(sourcepath+"/MFiles");
-  m_term->setPath(pth.toStdString());
-  m_term->setAppPath(qApp->applicationDirPath().toStdString());
-  WalkTree *twalk = new WalkTree(context,m_term);
+Interpreter* GetInterpreter(QStringList cmds) {
+  QString pth(sourcepath+"/toolbox");
+  QStringList pthList(GetRecursiveDirList(pth));
+  Interpreter *twalk = new Interpreter(context);
+  m_hlp = new HelpHandler(twalk,cmds);
+  twalk->setUserPath(pthList);
+  twalk->rescanPath();
+  twalk->AutoStop(false);
+  twalk->setGreetingFlag(true);
+  qRegisterMetaType<string>("string");
+  qRegisterMetaType<FuncPtr>("FuncPtr");
+  qRegisterMetaType<ArrayVector>("ArrayVector");
+  QObject::connect(twalk,SIGNAL(outputRawText(string)),
+		   m_hlp,SLOT(AddOutputString(string)));
+  QObject::connect(twalk,SIGNAL(SetPrompt(string)),
+		   m_hlp,SLOT(SetPrompt(string)));
+  QObject::connect(twalk,SIGNAL(doGraphicsCall(FuncPtr,ArrayVector,int)),
+		   m_hlp,SLOT(DoGraphicsCall(FuncPtr,ArrayVector,int)));
+  twalk->setTerminalWidth(80);
+  m_interp = twalk;
   return twalk;
 }
 
@@ -220,6 +225,7 @@ class HTMLWriter : public HelpWriter {
   QStringList eqnlist;
   bool verbatim;
   QString modulename;
+  bool ignore;
  public:
   HTMLWriter() {myfile = NULL; mystream = NULL;}
   virtual ~HTMLWriter() {}
@@ -294,11 +300,11 @@ void HTMLWriter::WriteIndex() {
   f << "\n";
   f << "<HTML>\n";
   f << "<HEAD>\n";
-  f << "<TITLE>" << WalkTree::getVersionString().c_str();
+  f << "<TITLE>" << Interpreter::getVersionString().c_str();
   f << " Documentation" << "</TITLE>\n";
   f << "</HEAD>\n";
   f << "<BODY>\n";
-  f << "<H1>" << WalkTree::getVersionString().c_str();
+  f << "<H1>" << Interpreter::getVersionString().c_str();
   f << " Documentation" << "</H1>\n";
   f << "<P>\n";
   f << "<H2> Documentation Sections </H2>\n";
@@ -419,7 +425,12 @@ void HTMLWriter::BeginModule(QString modname, QString moddesc, QString secname)
 }
 
 void HTMLWriter::BeginGroup(QString groupname) {
-  *mystream << "<H3>" << groupname << "</H3>\n";
+  if (groupname.toLower() == "tests")
+    ignore = true;
+  else {
+    *mystream << "<H3>" << groupname << "</H3>\n";
+    ignore = false;
+  }
 }
 
 void HTMLWriter::BeginVerbatim() {
@@ -442,6 +453,7 @@ QString LatinFilter(QString text) {
 }
 
 void HTMLWriter::OutputText(QString text) {
+  if (ignore) return;
   if (verbatim)
     *mystream << LatinFilter(text);
   else
@@ -478,6 +490,66 @@ void HTMLWriter::DoFile(QString fname, QString ftxt) {
   *mystream << "<P>\n";
 }
 
+class TestWriter : public HelpWriter {
+  QFile *myfile;
+  QTextStream *mystream;
+  QMultiMap<QString, QStringList> sectables;
+  bool verbatim;
+  bool ignore;
+  QString module;
+  int num;
+ public:
+  TestWriter();
+  virtual ~TestWriter() {}
+  void BeginModule(QString modname, QString moddesc, QString secname);
+  void BeginGroup(QString groupname);
+  void OutputText(QString text);
+  void WriteIndex();
+};
+
+
+void TestWriter::BeginModule(QString modname, QString moddesc, QString secname) {
+  module = modname;
+}
+
+TestWriter::TestWriter() {
+  myfile = new QFile(sourcepath+"/help/test/test_functions.m");
+  if (!myfile->open(QFile::WriteOnly))
+    Halt("Unable to open help/test/test_functions.m for output");
+  mystream = new QTextStream(myfile);
+  *mystream << "% Regression test function for " << Interpreter::getVersionString().c_str() << "\n";
+  *mystream << "% This function is autogenerated by helpgen.\n";
+  *mystream << "function test_functions\n";
+}
+
+void TestWriter::BeginGroup(QString groupname) {
+  if (groupname.toLower() != "tests") {
+    ignore = true;
+  } else {
+    ignore = false;
+  }
+  num = 1;
+}
+
+void TestWriter::OutputText(QString text) {
+  if (ignore) return;
+  QRegExp test_pattern("@\\{\"([^\"]*)\",\"([^\"]*)\",\"([^\"]*)\"\\}");
+  if (test_pattern.indexIn(text) < 0)
+    Halt("Bad line: " + text);
+  *mystream << "\n";
+  *mystream << test_pattern.cap(1) << ";\n";
+  *mystream << "q=" << test_pattern.cap(2) << ";\n";
+  *mystream << "test_" << test_pattern.cap(3) << "(q,y,'"<< module.toLower() << " " << num <<"');\n";
+  *mystream << "\n";
+  num++;
+}
+
+void TestWriter::WriteIndex() {
+  
+  delete mystream;
+  delete myfile;
+}
+
 class TextWriter : public HelpWriter {
   QFile *myfile;
   QTextStream *mystream;
@@ -516,7 +588,7 @@ void TextWriter::WriteIndex() {
   f << "\\oddsidemargin 0.0in\n";
   f << "\\evensidemargin 1.0in\n";
   f << "\\textwidth 6.0in\n";
-  f << "\\title{" << WalkTree::getVersionString().c_str() << " Documentation}\n";
+  f << "\\title{" << Interpreter::getVersionString().c_str() << " Documentation}\n";
   f << "\\author{Samit Basu}\n";
   f << "\\begin{document}\n";
   f << "\\maketitle\n";
@@ -560,6 +632,7 @@ void TextWriter::EndModule() {
 }
 
 void TextWriter::BeginModule(QString modname, QString moddesc, QString secname) {
+  qDebug() << "Module " << modname << " " << moddesc << " " << secname;
   moduledepth++;
   myfile = new QFile(sourcepath+"/help/text/" + modname.toLower() + ".mdc");
   if (!myfile->open(QFile::Append)) {
@@ -623,6 +696,7 @@ class LatexWriter : public HelpWriter {
   QTextStream *mystream;
   QMultiMap<QString, QStringList> sectables;
   bool verbatim;
+  bool ignore;
  public:
   LatexWriter() {myfile = NULL; mystream = NULL;}
   virtual ~LatexWriter() {}
@@ -654,7 +728,7 @@ void LatexWriter::WriteIndex() {
   f << "\\oddsidemargin 0.0in\n";
   f << "\\evensidemargin 1.0in\n";
   f << "\\textwidth 6.0in\n";
-  f << "\\title{" << WalkTree::getVersionString().c_str() << " Documentation}\n";
+  f << "\\title{" << Interpreter::getVersionString().c_str() << " Documentation}\n";
   f << "\\author{Samit Basu}\n";
   f << "\\begin{document}\n";
   f << "\\maketitle\n";
@@ -696,9 +770,10 @@ void LatexWriter::EndModule() {
 
 void LatexWriter::BeginModule(QString modname, QString moddesc, QString secname) {
   moduledepth++;
-  myfile = new QFile(sourcepath+"/help/latex/" + secname.toLower() + "_" + modname.toLower() + ".tex");
+  QString filename(sourcepath+"/help/latex/" + secname.toLower() + "_" + modname.toLower() + ".tex");
+  myfile = new QFile(filename);
   if (!myfile->open(QFile::WriteOnly)) {
-    Halt("Unable to open " + modname + ".tex for output " + QString().sprintf("%d",myfile->error()) + " depth = " + QString().sprintf("%d",moduledepth));
+    Halt("Unable to open " + filename + " for output " + QString().sprintf("%d",myfile->error()) + " depth = " + QString().sprintf("%d",moduledepth));
   }
   mystream = new QTextStream(myfile);
   *mystream << "\\section{" + moddesc + "}\n\n";
@@ -707,7 +782,12 @@ void LatexWriter::BeginModule(QString modname, QString moddesc, QString secname)
 }
 
 void LatexWriter::BeginGroup(QString groupname) {
-  *mystream << "\\subsection{" + groupname + "}\n\n";
+  if (groupname.toLower() == "tests")
+    ignore = true;
+  else {
+    ignore = false;
+    *mystream << "\\subsection{" + groupname + "}\n\n";
+  }
 }
 
 void LatexWriter::BeginVerbatim() {
@@ -721,6 +801,7 @@ QString LatexWriter::ExpandCodes(QString text) {
 }
 
 void LatexWriter::OutputText(QString text) {
+  if (ignore) return;
   if (verbatim)
     *mystream << text;
   else
@@ -747,36 +828,34 @@ void LatexWriter::DoFile(QString filename, QString ftext) {
   *mystream << "\\end{verbatim}\n";
 }
 
-using namespace FreeMat;
-
-
 QString EvaluateCommands(QStringList cmds, int expectedCount, QString modulename, QString file) {
-  input = cmds;
+  QString output;
+
   output.clear();
-  WalkTree* twalk = GetInterpreter();
-  try {
-    while (!input.empty()) {
-      try {
-	twalk->evalCLI();
-      } catch (WalkTreeRetallException) {
-	twalk->clearStacks();
-      } catch (WalkTreeReturnException &e) {
-      } catch (WalkTreeQuitException) {
-      }
-    }
-  } catch (WalkTreeQuitException &e) {
-  } catch (std::exception& e) {
-    std::cout << "Exception caught: " << e.what() << "\n";
-  }
+
+  QString cpath(QDir::currentPath());
+
+  //  qDebug() << "Evaluating:\n" << cmds;
+  Interpreter* twalk = GetInterpreter(cmds);
+  twalk->start();  
+  //   for (int i=0;i<cmds.size();i++)
+  //     twalk->ExecuteLine(cmds[i].toStdString());
+  //   // Need to 
+  while (!twalk->isFinished())
+    qApp->processEvents();
+  QDir::setCurrent(cpath);
+  output = QString::fromStdString(m_hlp->text());
   if (twalk->getErrorCount() != expectedCount) 
     Halt("Error: Got " + QString().sprintf("%d",twalk->getErrorCount()) + ", expected " + QString().sprintf("%d",expectedCount) + " in module " + modulename + " in file " + file + "\n\nOutput Follows:\n"+output);
+  //  cout << "Output: " << output.toStdString();
   ShutdownHandleGraphics();
+  //  cout << "Output: " << m_hlp->text();
   delete twalk;
-  delete m_term;
   delete m_hlp;
-  //  qDebug() << output;
   QRegExp mprintre("(--> mprint[^\\n]*)");
   output = output.replace(mprintre,"");
+  QRegExp quitre("(--> quit;)");
+  output = output.replace(quitre,"");
   return output;
 }
 
@@ -823,7 +902,7 @@ bool TestMatch(QRegExp re, QString source) {
 
 QString GetLine(QTextStream &strm) {
   QString wrk = strm.readLine(0);
-  std::string version(WalkTree::getVersionString());
+  std::string version(Interpreter::getVersionString());
   QString verstring(QString::fromStdString(version));
   int ndx = verstring.indexOf("v");
   verstring.remove(0,ndx+1);
@@ -853,7 +932,7 @@ void ProcessFile(QFileInfo fileinfo, HelpWriter *out) {
   QRegExp itemizeout_pattern;
   QRegExp item_pattern;
 
-  std::string version(WalkTree::getVersionString());
+  std::string version(Interpreter::getVersionString());
   QString verstring(QString::fromStdString(version));
   int ndx = verstring.indexOf("v");
   verstring.remove(0,ndx+1);
@@ -904,6 +983,11 @@ void ProcessFile(QFileInfo fileinfo, HelpWriter *out) {
     item_pattern = QRegExp("^\\s*%\\s*\\\\item(.*)");
   }
   context = new Context;
+  LoadModuleFunctions(context);
+  LoadClassFunction(context);
+  LoadCoreFunctions(context);
+  LoadFNFunctions(context);
+  LoadHandleGraphicsFunctions(context);  
   modulename_pattern.setCaseSensitivity(Qt::CaseInsensitive);
   moduledesc_pattern.setCaseSensitivity(Qt::CaseInsensitive);
   sectioname_pattern.setCaseSensitivity(Qt::CaseInsensitive);
@@ -1074,7 +1158,7 @@ void MergeFile(QFileInfo fileinfo) {
   // Open the file
   QFile h(fileinfo.absoluteFilePath());
   QString newname(fileinfo.absoluteFilePath());
-  int p = newname.indexOf("mfiles",0,Qt::CaseInsensitive);
+  int p = newname.indexOf("toolbox",0,Qt::CaseInsensitive);
   newname = newname.remove(0,p);
   newname = sourcepath+"/help/"+newname;
   // Create the directory structure if necessary
@@ -1175,26 +1259,30 @@ void ConsoleWidget::Run() {
   DeleteDirectory(sourcepath+"/help/tmp");
   DeleteDirectory(sourcepath+"/help/latex");
   DeleteDirectory(sourcepath+"/help/text");
-  DeleteDirectory(sourcepath+"/help/mfiles");
+  DeleteDirectory(sourcepath+"/help/test");
+  DeleteDirectory(sourcepath+"/help/toolbox");
   dir.mkpath(sourcepath+"/help/html");
   dir.mkpath(sourcepath+"/help/tmp");
   dir.mkpath(sourcepath+"/help/latex");
   dir.mkpath(sourcepath+"/help/text");
-  dir.mkpath(sourcepath+"/help/mfiles");
+  dir.mkpath(sourcepath+"/help/test");
+  dir.mkpath(sourcepath+"/help/toolbox");
   ReadSectionDescriptors();
   LatexWriter texout;
   HTMLWriter htmlout;
   GroupWriter out;
   TextWriter txtout;
+  TestWriter tstout;
   out.RegisterWriter(&texout);
   out.RegisterWriter(&htmlout);
   out.RegisterWriter(&txtout);
+  out.RegisterWriter(&tstout);
+  ProcessDir(QDir(sourcepath+"/toolbox"),&out,true); 
   ProcessDir(QDir(sourcepath+"/libs"),&out,true); 
   ProcessDir(QDir(sourcepath+"/help"),&out,false); 
-  ProcessDir(QDir(sourcepath+"/MFiles"),&out,true); 
-  MergeMFiles(QDir(sourcepath+"/MFiles"));
+  MergeMFiles(QDir(sourcepath+"/toolbox"));
   out.WriteIndex();
-  std::string version(WalkTree::getVersionString());
+  std::string version(Interpreter::getVersionString());
   QString verstring(QString::fromStdString(version));
   int ndx = verstring.indexOf("v");
   verstring.remove(0,ndx+1);
