@@ -20,6 +20,11 @@
 #define __helpgen_hpp__
 
 #include <QWidget>
+#include <string>
+#include "Array.hpp"
+#include "FunctionDef.hpp"
+
+using namespace std;
 
 class ConsoleWidget : public QWidget {
   Q_OBJECT
@@ -32,13 +37,21 @@ public:
   virtual ~ConsoleWidget() {};
 };
 
-class OutputHelper : public QObject {
+class Interpreter;
+
+class HelpHandler : public QObject {
   Q_OBJECT
+  Interpreter *m_interp;  
+  string m_text;
+  QStringList m_cmds;
 public:
-  OutputHelper() : QObject() {}
-private slots:
-  void AddOutputString(std::string);
-  void MoveDown();
+  HelpHandler(Interpreter *interp, QStringList cmds);
+  string text() {return m_text;
+  }
+public slots:
+  void AddOutputString(string);
+  void SetPrompt(string);
+  void DoGraphicsCall(FuncPtr f, ArrayVector m, int narg);
 };
 
 void DoHelpGen(QString src, QString dst);
